# -*- coding: utf-8 -*-

import hashlib
from base64 import urlsafe_b64encode

from .compat import iteritems
from .compat import urlparse
from .compat import quote
from .compat import b

from ._version import __version__


class UrlHelper(object):
    """
    Helper class to create single domain imgix URLs. Example:

      >>> str(UrlHelper('demos.imgix.net', '/bridge.png', params={'w': 100}))
      https://demos.imgix.net/bridge.png?w=100

    Parameters
    ----------
    domain : str
    path : str
    scheme : { 'https', 'http' }
    sign_key : str or None
        When provided, this key will be used to sign the generated image URLs.
        You can read more about URL signing on our docs:
        https://docs.imgix.com/setup/securing-images
    include_library_param : bool
        If `True`, each created URL is suffixed with 'ixlib' parameter
        indicating the library used for generating the URLs. (default `True`)
    params : dict
        Dictionary specifying URL parameters. Non-imgix parameters are
        added to the URL unprocessed. For a complete list of imgix
        supported parameters, visit https://docs.imgix.com/apis/url .
        (default {})

    Methods
    -------
    set_parameter(key, value)
    delete_parameter(key)
    """
    def __init__(
            self,
            domain,
            path,
            scheme="https",
            sign_key=None,
            include_library_param=True,
            params={}):

        self._scheme = scheme
        self._host = domain
        self._path = path
        self._sign_key = sign_key
        self._include_library_param = include_library_param
        self._parameters = {}

        for key, value in iteritems(params):
            self.set_parameter(key, value)

    @classmethod
    def from_url(cls, url):
        pass

    def set_parameter(self, key, value):
        """
        Set a url parameter.

        Parameters
        ----------
        key : str
            If key ends with '64', the value provided will be automatically
            base64 encoded.
        """
        if value is None or isinstance(value, (int, float, bool)):
            value = str(value)

        if key.endswith('64'):
            value = urlsafe_b64encode(value.encode('utf-8'))
            value = value.replace(b('='), b(''))

        self._parameters[key] = value

    def delete_parameter(self, key):
        """
        Deletes the value associated with `key` from recorded parameters.

        Parameters
        ----------
        key : str

        Raises
        ------
        KeyError
            If key doesn't exist in recorded parameters.
        """
        if key in self._parameters:
            del self._parameters[key]

    def _str_is_ascii(self, s):
        try:
            b(s).decode('ascii')
            return True
        except Exception:
            return False

    def __str__(self):
        """
        Generate URL from the recorded parameters.

        Returns
        -------
        str
        """
        query = {}

        for key in self._parameters:
            query[key] = self._parameters[key]

        path = self._path

        if self._include_library_param:
            query["ixlib"] = "python-" + __version__

        if path.startswith("http"):
            try:
                path = quote(path, safe="~()*!.'")
            except KeyError:
                path = quote(path.encode('utf-8'), safe="~()*!.'")

        if not path.startswith("/"):
            path = "/" + path  # Fix web proxy style URLs

        if not path.startswith("/http") and not self._str_is_ascii(path):
            try:
                path = quote(path)
            except KeyError:
                path = quote(path.encode('utf-8'))

        query = "&".join(
            (quote(key, "") + "=" + quote(query[key], ""))
            for key in sorted(query))

        if self._sign_key:
            delim = "" if query == "" else "?"
            signing_value = self._sign_key + path + delim + query
            signature = hashlib.md5(signing_value.encode('utf-8')).hexdigest()
            if query:
                query += "&s=" + signature
            else:
                query = "s=" + signature

        return urlparse.urlunparse([
            self._scheme,
            self._host,
            path,
            "",
            query,
            "", ])



# -*- coding: utf-8 -*-

import imgix

from imgix.compat import urlparse
from imgix.urlhelper import UrlHelper


def test_create():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png')
    assert type(helper) is UrlHelper


def test_create_with_url_parameters():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       include_library_param=False,
                       params={"w": 400, "h": 300})
    assert str(helper) == "https://my-social-network.imgix.net/users/1.png?" \
                          "h=300&w=400"


def test_create_with_splatted_falsy_parameter():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       include_library_param=False,
                       params={"or": 0})
    assert str(helper) == "https://my-social-network.imgix.net" \
                          "/users/1.png?or=0"


def test_create_with_signature():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       sign_key="FOO123bar",
                       include_library_param=False)
    assert str(helper) == \
        "https://my-social-network.imgix.net/users/1.png" \
        "?s=6797c24146142d5b40bde3141fd3600c"


def test_create_with_paremeters_and_signature():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       sign_key="FOO123bar",
                       include_library_param=False,
                       params={"w": 400, "h": 300})
    assert str(helper) == \
        "https://my-social-network.imgix.net/users/1.png" \
        "?h=300&w=400&s=1a4e48641614d1109c6a7af51be23d18"


def test_create_with_fully_qualified_url():
    helper = UrlHelper("my-social-network.imgix.net",
                       "http://avatars.com/john-smith.png",
                       sign_key="FOO123bar",
                       include_library_param=False)
    assert str(helper) == \
        "https://my-social-network.imgix.net/"\
        "http%3A%2F%2Favatars.com%2Fjohn-smith.png" \
        "?s=493a52f008c91416351f8b33d4883135"


def test_create_with_fully_qualified_url_with_special_chars():
    helper = UrlHelper("my-social-network.imgix.net",
                       u"http://avatars.com/でのパ.png",
                       sign_key="FOO123bar",
                       include_library_param=False)
    assert str(helper) == "https://my-social-network.imgix.net/http%3A%2F%2F" \
                          "avatars.com%2F%E3%81%A7%E3%81%AE%E3%83%91.png" \
                          "?s=8e04a5dd9a659a6a540d7c817d3df1d3"


def test_use_https():
    # Defaults to https
    helper = UrlHelper("my-social-network.imgix.net", "/users/1.png")
    assert urlparse.urlparse(str(helper)).scheme == "https"

    helper = UrlHelper('my-social-network.imgix.net', "/users/1.png",
                       scheme="http")
    assert urlparse.urlparse(str(helper)).scheme == "http"


def test_utf_8_characters():
    helper = UrlHelper('my-social-network.imgix.net', u'/ǝ',
                       include_library_param=False)
    assert str(helper) == "https://my-social-network.imgix.net/%C7%9D"


def test_more_involved_utf_8_characters():
    helper = UrlHelper('my-social-network.imgix.net',
                       u'/üsers/1/でのパ.png',
                       include_library_param=False)
    assert str(helper) == 'https://my-social-network.imgix.net/' \
                          '%C3%BCsers/1/%E3%81%A7%E3%81%AE%E3%83%91.png'


def test_param_values_are_escaped():
    helper = UrlHelper('my-social-network.imgix.net', 'demo.png',
                       params={"hello world": "interesting"},
                       include_library_param=False)

    assert str(helper) == "https://my-social-network.imgix.net/demo.png?" \
                          "hello%20world=interesting"


def test_param_keys_are_escaped():
    params = {"hello_world": "/foo\"> <script>alert(\"hacked\")</script><"}
    helper = UrlHelper('my-social-network.imgix.net', 'demo.png',
                       params=params,
                       include_library_param=False)

    assert str(helper) == "https://my-social-network.imgix.net/demo.png?" \
        "hello_world=%2Ffoo%22%3E%20%3Cscript%3Ealert%28%22" \
        "hacked%22%29%3C%2Fscript%3E%3C"


def test_base64_param_variants_are_base64_encoded():
    params = {"txt64": u"I cannøt belîév∑ it wors! 😱"}
    helper = UrlHelper('my-social-network.imgix.net', '~text',
                       params=params,
                       include_library_param=False)

    assert str(helper) == "https://my-social-network.imgix.net/~text?txt64=" \
        "SSBjYW5uw7h0IGJlbMOuw6l24oiRIGl0IHdvcu-jv3MhIPCfmLE"


def test_signing_url_with_ixlib():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png')
    assert str(helper) == (
        "https://my-social-network.imgix.net/users/1.png?ixlib=python-" +
        imgix._version.__version__)


def test_set_parameter():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       include_library_param=False)

    helper.set_parameter('w', 400)
    assert str(helper) == "https://my-social-network.imgix.net/" \
                          "users/1.png?w=400"

    helper.set_parameter('h', 300)
    assert str(helper) == "https://my-social-network.imgix.net/" \
                          "users/1.png?h=300&w=400"


def test_set_parameter_with_init_params():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       params={"or": 0},
                       include_library_param=False)

    helper.set_parameter('w', 400)
    helper.set_parameter('h', 300)
    assert str(helper) == "https://my-social-network.imgix.net" \
                          "/users/1.png?h=300&or=0&w=400"


def test_set_parameter_base64_encoded():
    helper = UrlHelper('my-social-network.imgix.net', '~text',
                       include_library_param=False)

    helper.set_parameter("txt64", u"I cannøt belîév∑ it wors! 😱")
    assert str(helper) == "https://my-social-network.imgix.net/~text?txt64=" \
                          "SSBjYW5uw7h0IGJlbMOuw6l24oiRIGl0IHdvcu-jv3MhIPCfmLE"


def test_set_parameter_with_none_value():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       params={'h': 300, 'w': 400},
                       include_library_param=False)

    helper.set_parameter("w", None)
    assert str(helper) == "https://my-social-network.imgix.net" \
                          "/users/1.png?h=300&w=None"


def test_set_parameter_with_false_value():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       params={'h': 300, 'w': 400},
                       include_library_param=False)

    helper.set_parameter("w", False)
    assert str(helper) == "https://my-social-network.imgix.net" \
                          "/users/1.png?h=300&w=False"


def test_delete_parameter():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       params={'h': 300, 'w': 400},
                       include_library_param=False)

    helper.delete_parameter('w')
    assert str(helper) == "https://my-social-network.imgix.net" \
                          "/users/1.png?h=300"


def test_delete_all_parameters():
    helper = UrlHelper('my-social-network.imgix.net', '/users/1.png',
                       params={'h': 300, 'w': 400},
                       include_library_param=False)

    helper.delete_parameter('w')
    helper.delete_parameter('h')
    assert str(helper) == "https://my-social-network.imgix.net/users/1.png"



# -*- coding: utf-8 -*-

import re

from .urlhelper import UrlHelper

from .constants import DOMAIN_PATTERN, SRCSET_TARGET_WIDTHS


SRCSET_DPR_TARGET_RATIOS = range(1, 6)


class UrlBuilder(object):
    """
    Create imgix URLs

    The URL builder can be reused to create URLs for any images on the
    provided domain.

    Parameters
    ----------
    domain : str
        Domain to use while creating imgix URLs.
    use_https : bool
        If `True`, create HTTPS imgix image URLs. (default `True`)
    sign_key : str or None
        When provided, this key will be used to sign the generated image URLs.
        You can read more about URL signing on our docs:
        https://docs.imgix.com/setup/securing-images
    include_library_param : bool
        If `True`, each created URL is suffixed with 'ixlib' parameter
        indicating the library used for generating the URLs. (default `True`)

    Methods
    -------
    validate_domain(domain)
        Returns true if the supplied string parameter pattern matches a valid
        domain name accepted by imgix
    create_url(path, params=None)
        Create URL with the supplied path and `params` parameters dict.
    create_srcset(path, params=None)
        Create srcset attribute value with the supplied path and
        `params` parameters dict.
        Will generate a fixed-width DPR srcset if a width OR height and aspect
        ratio are passed in as parameters. Otherwise will generate a srcset
        with width-descriptor pairs.
    """
    def __init__(
            self,
            domain,
            use_https=True,
            sign_key=None,
            include_library_param=True):

        self.validate_domain(domain)

        self._domain = domain
        self._sign_key = sign_key
        self._use_https = use_https
        self._include_library_param = include_library_param

    def validate_domain(self, domain):
        """
        Returns true if the supplied string parameter pattern matches a valid
        domain name accepted by imgix

        Parameters
        ----------
        domain : str

        Returns
        -------
        bool
        """

        err_str = str(
            'Domain must be passed in as fully-qualified domain names and ' +
            'should not include a protocol or any path element, i.e. ' +
            '"example.imgix.net".')

        if re.match(DOMAIN_PATTERN, domain) is None:
            raise ValueError(err_str)

    def create_url(self, path, params=None):
        """
        Create URL with supplied path and `params` parameters dict.

        Parameters
        ----------
        path : str
        params : dict
            Dictionary specifying URL parameters. Non-imgix parameters are
            added to the URL unprocessed. For a complete list of imgix
            supported parameters, visit https://docs.imgix.com/apis/url .
            (default None)

        Returns
        -------
        str
            imgix URL
        """
        if not params:
            params = {}

        domain = self._domain
        scheme = "https" if self._use_https else "http"

        url_obj = UrlHelper(
            domain,
            path,
            scheme,
            sign_key=self._sign_key,
            include_library_param=self._include_library_param,
            params=params)

        return str(url_obj)

    def create_srcset(self, path, params=None):
        """
        Create srcset attribute value with the supplied path and
        `params` parameters dict.
        Will generate a fixed-width DPR srcset if a width OR height and aspect
        ratio are passed in as parameters. Otherwise will generate a srcset
        with width-descriptor pairs.

        Parameters
        ----------
        path : str
        params : dict
            Dictionary specifying URL parameters. Non-imgix parameters are
            added to the URL unprocessed. For a complete list of imgix
            supported parameters, visit https://docs.imgix.com/apis/url .
            (default None)

        Returns
        -------
        str
            srcset attribute value
        """
        if not params:
            params = {}

        width = params['w'] if 'w' in params else None
        height = params['h'] if 'h' in params else None
        aspect_ratio = params['ar'] if 'ar' in params else None

        if (width or (height and aspect_ratio)):
            return self._build_srcset_DPR(path, params)
        else:
            return self._build_srcset_pairs(path, params)

    def _build_srcset_pairs(self, path, params):
        srcset = ''

        for i in range(len(SRCSET_TARGET_WIDTHS)):
            current_width = SRCSET_TARGET_WIDTHS[i]
            current_params = params
            current_params['w'] = current_width
            srcset += self.create_url(path, current_params) \
                + ' ' + str(current_width) + 'w,\n'

        return srcset[0:-2]

    def _build_srcset_DPR(self, path, params):
        srcset = ''

        for i in range(len(SRCSET_DPR_TARGET_RATIOS)):
            current_ratio = SRCSET_DPR_TARGET_RATIOS[i]
            current_params = params
            current_params['dpr'] = i+1
            srcset += self.create_url(path, current_params) \
                + ' ' + str(current_ratio) + 'x,\n'

        return srcset[0:-2]



# -*- coding: utf-8 -*-

import imgix
import hashlib
import re


def _default_srcset(params={}):
    ub = imgix.UrlBuilder('testing.imgix.net',
                          sign_key='MYT0KEN',
                          include_library_param=False)
    return ub.create_srcset('image.jpg', params)


def _parse_width(width):
    return int(width[0:-1])


def test_no_parameters_generates_srcset_pairs():
    srcset = _default_srcset()
    expected_number_of_pairs = 31
    assert(expected_number_of_pairs == len(srcset.split(',')))


def test_srcset_pair_values():
    # array of expected resolutions to be generated
    resolutions = [100, 116, 134, 156, 182, 210, 244, 282,
                   328, 380, 442, 512, 594, 688, 798, 926,
                   1074, 1246, 1446, 1678, 1946, 2258, 2618,
                   3038, 3524, 4088, 4742, 5500, 6380, 7400, 8192]
    srcset = _default_srcset()
    srclist = srcset.split(',')
    index = 0

    for src in srclist:
        width = src.split(' ')[1]

        # extract width int values
        value = re.search(re.compile(r'\d+'), width)
        assert(int(value.group(0)) == resolutions[index])
        index += 1


def test_given_width_srcset_is_DPR():
    srcset = _default_srcset({'w': 100})
    device_pixel_ratio = 1
    srclist = srcset.split(',')

    for src in srclist:
        ratio = src.split(' ')[1]
        dpr_str = str(device_pixel_ratio) + 'x'
        assert(dpr_str == ratio)
        device_pixel_ratio += 1


def test_given_width_srcset_has_dpr_params():
    srcset = _default_srcset({'w': 100})
    srclist = srcset.split(',')

    for i in range(len(srclist)):
        src = srclist[i].split(' ')[0]
        assert(src.index("dpr=" + str(i+1)))


def test_given_width_signs_urls():
    srcset = _default_srcset({'w': 100})
    srclist = srcset.split(',')

    for src in srclist:
        url = src.split(' ')[0]
        assert('s=' in url)

        # extract the sign parameter
        generated_signature = url[url.index('s=')+2:len(url)]

        params = url[url.index('?'):url.index('s=')-1]
        signature_base = 'MYT0KEN' + '/image.jpg' + params
        expected_signature = hashlib.md5(signature_base
                                         .encode('utf-8')).hexdigest()

        assert(expected_signature == generated_signature)


def test_given_height_srcset_generates_pairs():
    srcset = _default_srcset({'h': 100})
    expected_number_of_pairs = 31
    assert(expected_number_of_pairs == len(srcset.split(',')))


def test_given_height_respects_parameter():
    srcset = _default_srcset({'h': 100})
    srclist = srcset.split(',')

    for src in srclist:
        assert('h=100' in src)


def test_given_height_srcset_pairs_within_bounds():
    srcset = _default_srcset({'h': 100})
    srclist = srcset.split(',')

    min_parsed = srclist[0].split(' ')[1]
    max_parsed = srclist[-1].split(' ')[1]
    min = _parse_width(min_parsed)
    max = _parse_width(max_parsed)

    assert(min >= 100)
    assert(max <= 8192)


# a 17% testing threshold is used to account for rounding
def test_given_height_srcset_iterates_17_percent():
    increment_allowed = 0.17
    srcset = _default_srcset({'h': 100})
    srcslist = srcset.split(',')
    widths_list = [src.split(' ')[1] for src in srcslist]
    # a list of all widths as integers
    widths = [_parse_width(width) for width in widths_list]

    prev = widths[0]
    for i in range(1, len(widths)):
        width = widths[i]
        assert((width / prev) < (1 + increment_allowed))
        prev = width


def test_given_height_srcset_signs_urls():
    srcset = _default_srcset({'h': 100})
    srclist = srcset.split(',')
    srcs = [src.split(' ')[0] for src in srclist]

    for src in srcs:
        assert(src.index('s='))
        params = src[src.index('?'):len(src)]
        params = params[0:params.index('s=')-1]
        # extract the sign parameter
        generated_signature = src[src.index('s=')+2:len(src)]

        signature_base = 'MYT0KEN' + '/image.jpg' + params
        expected_signature = hashlib.md5(signature_base
                                         .encode('utf-8')).hexdigest()

        assert(expected_signature == generated_signature)


def test_given_width_and_height_is_DPR():
    srcset = _default_srcset({'w': 100, 'h': 100})
    device_pixel_ratio = 1
    srclist = srcset.split(',')

    for src in srclist:
        ratio = src.split(' ')[1]
        dpr_str = str(device_pixel_ratio) + 'x'
        assert(dpr_str == ratio)
        device_pixel_ratio += 1


def test_given_width_and_height_srcset_has_dpr_params():
    srcset = _default_srcset({'w': 100, 'h': 100})
    srclist = srcset.split(',')

    for i in range(len(srclist)):
        src = srclist[i].split(' ')[0]
        assert(src.index("dpr=" + str(i+1)))


def test_given_width_and_height_signs_urls():
    srcset = _default_srcset({'w': 100, 'h': 100})
    srclist = srcset.split(',')

    for src in srclist:
        url = src.split(' ')[0]
        assert('s=' in url)

        # extract the sign parameter
        generated_signature = url[url.index('s=')+2:len(url)]

        params = url[url.index('?'):url.index('s=')-1]
        signature_base = 'MYT0KEN' + '/image.jpg' + params
        expected_signature = hashlib.md5(signature_base
                                         .encode('utf-8')).hexdigest()

        assert(expected_signature == generated_signature)


def test_given_aspect_ratio_srcset_generates_pairs():
    srcset = _default_srcset({'ar': '3:2'})
    expected_number_of_pairs = 31
    assert(expected_number_of_pairs == len(srcset.split(',')))


def test_given_aspect_ratio_srcset_pairs_within_bounds():
    srcset = _default_srcset({'ar': '3:2'})
    srclist = srcset.split(',')

    min_parsed = srclist[0].split(' ')[1]
    max_parsed = srclist[-1].split(' ')[1]
    min = _parse_width(min_parsed)
    max = _parse_width(max_parsed)

    assert(min >= 100)
    assert(max <= 8192)


# a 17% testing threshold is used to account for rounding
def test_given_aspect_ratio_srcset_iterates_17_percent():
    increment_allowed = 0.17
    srcset = _default_srcset({'ar': '3:2'})
    srcslist = srcset.split(',')
    widths_list = [src.split(' ')[1] for src in srcslist]
    # a list of all widths as int
    widths = [_parse_width(width) for width in widths_list]

    prev = widths[0]
    for i in range(1, len(widths)):
        width = widths[i]
        assert((width / prev) < (1 + increment_allowed))
        prev = width


def test_given_aspect_ratio_srcset_signs_urls():
    srcset = _default_srcset({'ar': '3:2'})
    srclist = srcset.split(',')
    srcs = [src.split(' ')[0] for src in srclist]

    for src in srcs:
        assert(src.index('s='))
        params = src[src.index('?'):len(src)]
        params = params[0:params.index('s=')-1]
        # extract the sign parameter
        generated_signature = src[src.index('s=')+2:len(src)]

        signature_base = 'MYT0KEN' + '/image.jpg' + params
        expected_signature = hashlib.md5(signature_base
                                         .encode('utf-8')).hexdigest()

        assert(expected_signature == generated_signature)


def test_given_aspect_ratio_and_height_srcset_is_DPR():
    srcset = _default_srcset({'ar': '3:2', 'h': 500})
    device_pixel_ratio = 1
    srclist = srcset.split(',')

    for src in srclist:
        ratio = src.split(' ')[1]
        dpr_str = str(device_pixel_ratio) + 'x'
        assert(dpr_str == ratio)
        device_pixel_ratio += 1


def test_given_aspect_ratio_and_height_srcset_has_dpr_params():
    srcset = _default_srcset({'ar': '3:2', 'h': 500})
    srclist = srcset.split(',')

    for i in range(len(srclist)):
        src = srclist[i].split(' ')[0]
        assert(src.index("dpr=" + str(i+1)))


def test_given_aspect_ratio_and_height_srcset_signs_urls():
    srcset = _default_srcset({'ar': '3:2', 'h': 500})
    srclist = srcset.split(',')

    for src in srclist:
        url = src.split(' ')[0]
        assert('s=' in url)

        # extract the sign parameter
        generated_signature = url[url.index('s=')+2:len(url)]

        params = url[url.index('?'):url.index('s=')-1]
        signature_base = 'MYT0KEN' + '/image.jpg' + params
        expected_signature = hashlib.md5(signature_base
                                         .encode('utf-8')).hexdigest()

        assert(expected_signature == generated_signature)



import re

DOMAIN_PATTERN = re.compile(
            r'^(?:[a-z\d\-_]{1,62}\.){0,125}'
            r'(?:[a-z\d](?:\-(?=\-*[a-z\d])|[a-z]|\d){0,62}\.)'
            r'[a-z\d]{1,63}$'
        )
SRCSET_INCREMENT_PERCENTAGE = 8
SRCSET_MAX_SIZE = 8192


def _target_widths():
    resolutions = []
    prev = 100

    def ensure_even(n):
        return 2 * round(n/2.0)

    while prev <= SRCSET_MAX_SIZE:
        resolutions.append(int(ensure_even(prev)))
        prev *= 1 + (SRCSET_INCREMENT_PERCENTAGE / 100.0) * 2

    resolutions.append(SRCSET_MAX_SIZE)
    return resolutions


SRCSET_TARGET_WIDTHS = _target_widths()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs

from imgix._version import __version__
from setuptools import setup

with codecs.open('README.rst', encoding='utf-8') as fp:
    readme = fp.read()
with codecs.open('CHANGELOG.rst', encoding='utf-8') as fp:
    changelog = fp.read()


setup(
    name='imgix',
    version=__version__,
    author='imgix',
    author_email='support@imgix.com',
    packages=['imgix'],
    url='https://github.com/imgix/imgix-python',
    license='BSD-2-Clause',
    description='Python client library for imgix.',
    long_description=u'\n\n'.join([readme, changelog]),
    long_description_content_type=u'text/x-rst',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    setup_requires=['pytest-runner'],
    tests_require=[
        'pytest',
        'pytest-cov',
    ],
    )



# -*- coding: utf-8 -*-

import imgix

from imgix.compat import urlparse


def _get_domain(url):
    return urlparse.urlparse(url).hostname


def _default_builder():
    return imgix.UrlBuilder('my-social-network.imgix.net',
                            include_library_param=False)


def _default_builder_with_signature():
    return imgix.UrlBuilder('my-social-network.imgix.net', True, "FOO123bar",
                            include_library_param=False)


def test_create():
    builder = imgix.UrlBuilder('my-social-network.imgix.net')
    assert type(builder) is imgix.UrlBuilder


def test_create_accepts_domains_single_str():
    domain = 'my-social-network-1.imgix.net'
    builder = imgix.UrlBuilder(domain)
    assert domain == _get_domain(builder.create_url('/users/1.png'))


def test_create_url_with_path():
    builder = _default_builder()
    url = builder.create_url("/users/1.png")
    assert url == "https://my-social-network.imgix.net/users/1.png"


def test_create_url_with_path_and_parameters():
    builder = _default_builder()
    url = builder.create_url("/users/1.png", {"w": 400, "h": 300})
    assert url == "https://my-social-network.imgix.net/users/1.png?h=300&w=400"


def test_create_url_with_params_kwarg():
    builder = _default_builder()
    url = builder.create_url("/users/1.png", params={"w": 400, "h": 300})
    assert url == "https://my-social-network.imgix.net/users/1.png?h=300&w=400"


def test_create_url_with_splatted_falsy_parameter():
    builder = _default_builder()
    url = builder.create_url("/users/1.png", {"or": 0})
    assert url == "https://my-social-network.imgix.net/users/1.png?or=0"


def test_create_url_with_path_and_signature():
    builder = _default_builder_with_signature()
    url = builder.create_url("/users/1.png")
    assert url == \
        "https://my-social-network.imgix.net/users/1.png" \
        "?s=6797c24146142d5b40bde3141fd3600c"


def test_create_url_with_path_and_paremeters_and_signature():
    builder = _default_builder_with_signature()
    url = builder.create_url("/users/1.png", {"w": 400, "h": 300})
    assert url == \
        "https://my-social-network.imgix.net/users/1.png" \
        "?h=300&w=400&s=1a4e48641614d1109c6a7af51be23d18"


def test_create_url_with_fully_qualified_url():
    builder = _default_builder_with_signature()
    url = builder.create_url("http://avatars.com/john-smith.png")
    assert url == \
        "https://my-social-network.imgix.net/"\
        "http%3A%2F%2Favatars.com%2Fjohn-smith.png" \
        "?s=493a52f008c91416351f8b33d4883135"


def test_create_url_with_fully_qualified_url_with_tilde():
    builder = _default_builder()
    url = builder.create_url("http://avatars.com/john~smith.png")
    assert url == \
        "https://my-social-network.imgix.net/"\
        "http%3A%2F%2Favatars.com%2Fjohn~smith.png"


def test_create_url_with_fully_qualified_url_and_parameters():
    builder = _default_builder_with_signature()
    url = builder.create_url("http://avatars.com/john-smith.png",
                             {"w": 400, "h": 300})
    assert url == \
        "https://my-social-network.imgix.net/" \
        "http%3A%2F%2Favatars.com%2Fjohn-smith.png" \
        "?h=300&w=400&s=a201fe1a3caef4944dcb40f6ce99e746"


def test_use_https():
    # Defaults to https
    builder = imgix.UrlBuilder('my-social-network.imgix.net')
    url = builder.create_url("/users/1.png")
    assert urlparse.urlparse(url).scheme == "https"

    builder = imgix.UrlBuilder('my-social-network.imgix.net', use_https=False)
    url = builder.create_url("/users/1.png")
    assert urlparse.urlparse(url).scheme == "http"

    builder = imgix.UrlBuilder('my-social-network.imgix.net', True)
    url = builder.create_url("/users/1.png")
    assert urlparse.urlparse(url).scheme == "https"

    builder = imgix.UrlBuilder('my-social-network.imgix.net', use_https=True)
    url = builder.create_url("/users/1.png")
    assert urlparse.urlparse(url).scheme == "https"


def test_utf_8_characters():
    builder = _default_builder()
    url = builder.create_url(u'/ǝ')
    assert url == "https://my-social-network.imgix.net/%C7%9D"


def test_more_involved_utf_8_characters():
    builder = _default_builder()
    url = builder.create_url(u'/üsers/1/米国でのパーティーします。.png')
    assert url == \
        "https://my-social-network.imgix.net/%C3%BCsers/1/" \
        "%E7%B1%B3%E5%9B%BD%E3%81%A7%E3%81%AE%E3%83%91%E3%83%BC%E3%83" \
        "%86%E3%82%A3%E3%83%BC%E3%81%97%E3%81%BE%E3%81%99%E3%80%82.png"


def test_param_values_are_escaped():
    builder = _default_builder()
    url = builder.create_url('demo.png', {"hello world": "interesting"})

    assert url == "https://my-social-network.imgix.net/demo.png?" \
        "hello%20world=interesting"


def test_param_keys_are_escaped():
    builder = _default_builder()
    url = builder.create_url('demo.png', {
        "hello_world": "/foo\"> <script>alert(\"hacked\")</script><"})

    assert url == "https://my-social-network.imgix.net/demo.png?" \
        "hello_world=%2Ffoo%22%3E%20%3Cscript%3Ealert%28%22" \
        "hacked%22%29%3C%2Fscript%3E%3C"


def test_base64_param_variants_are_base64_encoded():
    builder = _default_builder()
    url = builder.create_url('~text', {
        "txt64": u"I cannøt belîév∑ it wors! 😱"})

    assert url == "https://my-social-network.imgix.net/~text?txt64=" \
        "SSBjYW5uw7h0IGJlbMOuw6l24oiRIGl0IHdvcu-jv3MhIPCfmLE"


def test_signing_url_with_ixlib():
    builder = imgix.UrlBuilder('my-social-network.imgix.net')
    url = builder.create_url("/users/1.png")
    assert url == (
        "https://my-social-network.imgix.net/users/1.png?ixlib=python-" +
        imgix._version.__version__)


def test_invalid_domain_append_slash():
    url_append_slash = 'assets.imgix.net/products'

    # Should fail if the expected error isn't raised
    try:
        imgix.UrlBuilder(url_append_slash)
    except ValueError:
        assert(True)
    else:
        assert(False)


def test_invalid_domain_prepend_scheme():
    url_prepend_protocol = 'https://assets.imgix.net'

    # Should fail if the expected error isn't raised
    try:
        imgix.UrlBuilder(url_prepend_protocol)
    except ValueError:
        assert(True)
    else:
        assert(False)


def test_invalid_domain_append_dash():
    url_append_dash = 'assets.imgix.net-products'

    # Should fail if the expected error isn't raised
    try:
        imgix.UrlBuilder(url_append_dash)
    except ValueError:
        assert(True)
    else:
        assert(False)


def test_include_library_param_true():
    url = ("https://assets.imgix.net/image.jpg?ixlib=python-" +
           imgix._version.__version__)
    ub = imgix.UrlBuilder("assets.imgix.net", include_library_param=True)

    assert url == ub.create_url("image.jpg")


def test_include_library_param_false():
    url = 'https://assets.imgix.net/image.jpg'
    ub = imgix.UrlBuilder("assets.imgix.net", include_library_param=False)

    assert url == ub.create_url("image.jpg")



# -*- coding: utf-8 -*-
"""
imgix
~~~~~

A Python client library for generating URLs with imgix. Basic usage:

    >>> import imgix
    >>> builder = imgix.UrlBuilder("demos.imgix.net")
    >>> builder.create_url("/bridge.png", {'w': 100, 'h': 100})
    https://demos.imgix.net/bridge.png?h=100&w=100

... or generating signed URLs:

    >>> builder = imgix.UrlBuilder("demos.imgix.net", sign_key="test1234")
    >>> builder.create_url("/bridge.png", {'w': 100, 'h': 100})
    http://demos.imgix.net/bridge.png?h=100&w=100&s=7370d6e36bb2262e73b19578739af1af

Refer to `imgix.UrlBuilder` class documentation for all the supported options.
"""

from ._version import __version__

from .urlbuilder import UrlBuilder

__all__ = [
    'UrlBuilder',
    'SIGNATURE_MODE_QUERY', 'SIGNATURE_MODE_PATH', '__version__', ]
